unit Check;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, Dialogs, SysUtils, ExtCtrls;

type
  TCheckDlg = class(TForm)
    OKBtn: TBitBtn;
    Panel1: TPanel;
    ErrorOutput: TMemo;
    procedure OKBtnClick(Sender: TObject);
    procedure CheckBoardOutput(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  CheckDlg: TCheckDlg;

implementation

uses Unit1;

{$R *.DFM}

procedure TCheckDlg.OKBtnClick(Sender: TObject);
begin
  ResetSel;
  Close;
end;

procedure ErrorTxt (i, j: byte; msg: string);
begin
  CheckDlg.ErrorOutput.Lines.Add ('[' + IntToStr (i) + '; '
                          + IntToStr (j) + ']: ' + msg);
end;

procedure CheckAdjacent (i, j, brk1, brk2: Byte; msg: String; var brError: byte);
begin
  if (be.bType [be.bID [i, j]] = brk1) then begin
    if (be.bType [be.bID [i, j+1]] = brk2) and
    (be.bID [i, j] <> be.bID [i, j+1]) and (j <> be.nTiles - 1) then begin
      ErrorTxt (i, j, msg);
      Inc (brError);
      sel [i, j] := 1;
      sel [i, j+1] := 1;
      DrawTile (i, j);
      DrawTile (i, j+1);
    end;
    if (be.bType [be.bID [i+1, j]] = brk2) and
    (be.bID [i, j] <> be.bID [i+1, j]) and (j <> be.nTiles - 1) then begin
      ErrorTxt (i, j, msg);
      Inc (brError);
      sel [i, j] := 1;
      sel [i+1, j] := 1;
      DrawTile (i, j);
      DrawTile (i+1, j);
    end;
  end;
end;

procedure TCheckDlg.CheckBoardOutput(Sender: TObject);
  var i, j: ShortInt;
      brError, keys: byte;
      mx, my, tx, ty: byte;
      Match: Boolean;
begin
  CheckDlg.ErrorOutput.Clear;
  CheckDlg.ErrorOutput.Scrollbars := ssNone;
  brError := 0;
  mx := 99; tx := 99;
  Keys := 0;
  for i := 0 to be.nTiles - 1 do
    for j := 0 to be.nTiles - 1 do begin
      (* Position von Master und Target *)
      if (be.bType [be.bID [i, j]] = brkMaster) and (mx = 99) then begin
        mx := i;
        my := j;
      end;
      if (be.bForm [i, j] and 192 = 64) and (tx = 99) then begin
        tx := i;
        ty := j;
      end;

      (* Eliminatoren und Magnete *)
      CheckAdjacent (i, j, brkElim, brkElim, 'Eliminators touch.', brError);
      CheckAdjacent (i, j, brkElim, brkWater, 'Eliminator touches water.', brError);
      CheckAdjacent (i, j, brkWater, brkElim, 'Eliminator touches water.', brError);
      CheckAdjacent (i, j, brkMagnet, brkMagnet, 'Magnets touch.', brError);
      CheckAdjacent (i, j, brkAnti, brkMagnet, 'Magnet and anti magnet touch.', brError);
      CheckAdjacent (i, j, brkMagnet, brkAnti, 'Magnet and anti magnet touch.', brError);
      CheckAdjacent (i, j, brkAnti, brkAnti, 'Anti magnets touch.', brError);

      (* Schlssel, Blockaden usw. *)
      if (be.bType [be.bID [i, j]] = brkKey) then
        keys := keys or 4;
      if (be.bType [be.bID [i, j]] = brkKeyhole) then
        keys := keys or 2;
      if (be.bType [be.bID [i, j]] = brkBlockade) then
        keys := keys or 1;
    end;

  case (keys) of
    1:    CheckDlg.ErrorOutput.Lines.Add ('There are blockade bricks, but no key.');
    2, 3: CheckDlg.ErrorOutput.Lines.Add ('Keyhole has no key.');
    4, 5: CheckDlg.ErrorOutput.Lines.Add ('Key has no keyhole.');
    6:    CheckDlg.ErrorOutput.Lines.Add ('There are a key and a keyhole, but no blockade bricks.');
  end;

  if (keys <> 0) and (keys <> 7) then Inc (brError);

  (* bereinstimmung von Master und Target *)
  if (mx = 99) then begin
    CheckDlg.ErrorOutput.Lines.Add ('Master brick is missing!');
    Inc (brError);
  end;
  if (tx = 99) then begin
    CheckDlg.ErrorOutput.Lines.Add ('Target is missing!');
    Inc (brError);
  end;
  if (tx <> 99) and (mx <> 99) then begin
    Match := True;
    for i := 0 to be.nTiles - 1 do
      for j := 0 to be.nTiles - 1 do begin
        if (be.bType [be.bID [i, j]] = brkMaster) and
        (be.bForm [i-mx+tx, j-my+ty] and 192 <> 64) then
          Match := False;
        if (be.bForm [i-mx+tx, j-my+ty] and 192 = 64) and
        (be.bType [be.bID [i, j]] <> brkMaster) and
        (i-mx+tx >= 0) and (j-my+ty >= 0) then
          Match := False;
      end;
    if not (Match) then begin
      CheckDlg.ErrorOutput.Lines.Add ('Master and Target don''t match!');
      Inc (brError);
    end else
      if (tx = mx) and (my = ty) then begin
        CheckDlg.ErrorOutput.Lines.Add ('Master and Target are coincident.');
        Inc (brError);
      end;
  end;

  if (brError = 0) then
    CheckDlg.ErrorOutput.Lines.Add ('The board setup has been verified as correct.')
  else begin
    CheckDlg.ErrorOutput.Lines.Add ('');
    if (brError = 1) then
    CheckDlg.ErrorOutput.Lines.Add ('The board was checked and '
      + 'one error was found.')
    else
    CheckDlg.ErrorOutput.Lines.Add ('The board was checked and '
      + IntToStr (brError) + ' errors were found.');

    if (brError > 9) then   
      CheckDlg.ErrorOutput.Scrollbars := ssVertical;
  end;
end;

end.
